/**
* libs v7.0.9 by sunsilent
* @license MIT
**/
/*! For license information please see index.js.LICENSE.txt */
/**
* libs v7.0.4 by sunsilent
* @license MIT
**/ function matchWords(options) {
    const { source, searchWords, caseSensitive } = options;
    if ('string' != typeof source) throw new Error(`${source} is not string type.`);
    const regexp = new RegExp(searchWords.map((e)=>`(${e})`).join('|'), caseSensitive ? 'g' : 'gi');
    const r = source.matchAll(regexp);
    const words = [];
    let d = r.next();
    while(!d.done){
        const start = d.value.index;
        const end = start + d.value[0].length;
        if (words.length) {
            if (words[words.length - 1].end !== start) words.push({
                start: words[words.length - 1].end,
                end: start,
                highlight: false,
                value: source.substring(words[words.length - 1].end, start)
            });
            else words.push({
                start,
                end,
                highlight: false,
                value: source.substring(start, end)
            });
        } else {
            if (0 !== start) words.push({
                start: 0,
                end: start,
                highlight: false,
                value: source.substring(0, start),
                index: 0
            });
            words.push({
                start,
                end,
                highlight: true,
                value: d.value[0]
            });
        }
        updateIndex(words);
        d = r.next();
    }
    if (words.length) {
        if (words[words.length - 1].end !== source.length - 1) {
            const start = words[words.length - 1].end;
            const end = words[words.length - 1].end + source.length - 1;
            words.push({
                start,
                end,
                highlight: false,
                value: source.substring(start, source.length)
            });
        }
    }
    return {
        values: words,
        searchWords
    };
}
function updateIndex(words) {
    let count = 0;
    const value = words[words.length - 1].value;
    words.forEach((e)=>{
        if (e.value === value) count += 1;
    });
    words[words.length - 1].index = count > 0 ? count - 1 : 0;
}
function generate(matchResult, config) {
    const { values } = matchResult;
    let res = '';
    values.forEach((val)=>{
        res += createHtml(val, config);
    });
    return res;
}
function createHtml(val, config) {
    const options = config[val.value];
    const { activeClassName, activeStyle, activeIndex, className, highlightClassName, highlightStyle, highlightTag } = options || {};
    const cls = `${className || ''} ${highlightClassName || ''} ${val.index === activeIndex ? activeClassName || '' : ''}`;
    const style = `${highlightStyle || ''} ${val.index === activeIndex ? activeStyle || '' : ''}`;
    if (!highlightTag) return val.value;
    return `<${highlightTag} class="${cls}" style="${style}">${val.value}</${highlightTag}>`;
}
function highlight(options) {
    const { source, searchWords, config } = options;
    return generate(matchWords({
        source,
        searchWords
    }), config);
}
export { createHtml, generate, highlight };
